-- Form.ex ----------------------------------------------------------------------
-- A simple demo program using a Form to input customer identification
-- records from the keyboard.
--
-- Sequences are used to represent:
--	1. a data form for customer IDs,
--	2. each customer ID itself, and
--	3. a list containing typed-in customer IDs.
--
-- The program also illustrates typical uses of euphoria functions and procedures.
-- -------------------------------------------------------------------------------
-- Fred Mangan 2010


include misc.e	-- for procedure pretty_print()
include get.e	 -- for function prompt_number()

	-- the basic form is declared here as a constant; in a real-life
	-- program it would probably be imported from a database.
	-- (a constant takes on the type of whatever it is initialled to,
	--  in this case a fairly complex sequence)
	constant CUST_ID_FORM =
						{-- prompt         field-length (in number of characters)
						 {  "Name",		   30},		-- for field 1
						 {  "Address-1",	  20},		-- for field 2
						 {  "Address-2",	  20},		-- etc ...
						 {  "City",		   15},
						 {  "State/County",   20},
						 {  "Post Code",       4},
						 {  "Country",		20},
						 {  "Customer_ID",     7}		 -- for field 8
						}

function TypeInACustomer(sequence cust_list, integer count, integer nrecs)
-- Accepts typed fields from keyboard using the form CUST_ID_FORM
-- and adds the customer to a list.
-- Without error checking! (a "real-life" program would need to check what user types).
-- Entry:	cust_list = list to which new customer is to be added.
--			   count = a count for this records' number.
--			   nrecs = the total number of records user will be typeing in.
-- Returns:  cust_list with the new customer appended to it.
	sequence prompt_dots, row_col, tmp_data, tmp_cust
	integer num_fields

	-- a heading
	clear_screen()
	printf(1, "DATA ENTRY -- CUST_ID_FORM: Record %d/%d",{count,nrecs})

	num_fields = length(CUST_ID_FORM)	-- get ready

	-- display the form on screen
	for i = 1 to num_fields do

		prompt_dots = repeat('.',CUST_ID_FORM[i][2])
		printf(1, "\n\n%-13s: %s", {CUST_ID_FORM[i][1],prompt_dots})

	end for

	-- get the typed-in customer information:
	-- do initialling:
	tmp_cust = {}				-- create temporary (nul) customer
	row_col = {3,16}			 -- for offsetting into displayed form

	-- get data for each of num_fields from the user:
	for field = 1 to num_fields do

		position(row_col[1],row_col[2])			   -- place cursor
		tmp_data = gets(0)							-- get typing input
		puts(1, '\n')
		tmp_cust = append(tmp_cust,tmp_data[1..$-1])  -- add it to tmp_cust
		row_col += {2,0}							  -- advance on screen to next field

	end for

	-- add the new customer to list
	cust_list = append(cust_list, tmp_cust)

	return cust_list	-- return the list
end function -- TypeInACustomer

function AddCustomerControl()
-- This function has overall control of the process of adding customers.
-- 	Asks user how many customer IDs to enter;
-- 	Calls function TypeInACustomer() the specified number of times.
-- Entry:	none
-- Returns:  A list of the customer IDs.
--
	sequence customer_list
	integer num_add

	-- get number of customers to add
	num_add = prompt_number("How many customers will you add? : ",{1,3})

	customer_list = {}	-- we start the list off as a null (empty) sequence

	for i = 1 to num_add do

		-- add a customer to the customer_list
		customer_list = TypeInACustomer(customer_list, i, num_add)

	end for

	return customer_list
end function -- AddCustomerControl

procedure ShowResult(sequence C_List)
-- Displays the C_List as a pretty_printed list on screen
-- A "real" world program would probably not use a procedure like this;
-- it would instead use a routine to store the data in a database.
	clear_screen()

	puts(1, "THIS IS THE CUSTOMER LIST:\n\n")

	pretty_print(1, C_List, "2")	 	  -- display it (in a debug style)

	puts(1, "\n\n")						-- move down 2 lines

	system("Pause",2)					  -- DOS pause command, so user can see list

end procedure -- ShowResult

procedure MainLine()
-- A mainline procedure like this typically does little itself
-- except call routines in the logical order of the program design.
-- In many ways it "tells the story" of what the program does.
--
	sequence CustomerList	-- to hold the user's typed data

	CustomerList = AddCustomerControl()	-- create customer list

	ShowResult(CustomerList)			   -- display what was typed in

end procedure -- MainLine


MainLine() -- this gets the program going






